const withCss = require('./plugins/with-css')
const withBundleAnalyzer = require('@next/bundle-analyzer')
const withOptimizedImages = require('next-optimized-images')
const withGraphqlBasic = require('./plugins/with-graphql-basic')
const withTMBase = require('next-transpile-modules')

// Export a plugin function that just goes through and calls our chain
module.exports = function withHashicorp(options = {}) {
  if (!options.css) options.css = {}
  if (!options.dato) options.dato = {}
  if (!options.transpileModules) options.transpileModules = []

  return function withHashicorpInternal(nextConfig = {}) {
    const chain = [
      withOptimizedImages,
      withBundleAnalyzer({ enabled: process.env.ANALYZE === 'true' }),
      withCss(options.css),
      withGraphqlBasic(),
    ]

    // If there are explicitly defined modules to be transpiled, add them to the array
    // Also, don't transpile modules from this lib
    if (options.transpileModules) {
      chain.unshift(
        withTMBase([
          ...options.transpileModules,
          '@hashicorp/nextjs-scripts',
          '@hashicorp/react-.*',
        ])
      )
    }

    // Set dato token if a custom token is provided
    if (options.dato.token) {
      if (!nextConfig.env) nextConfig.env = {}
      process.env.HASHI_DATO_TOKEN = options.dato.token
      nextConfig.env.HASHI_DATO_TOKEN = options.dato.token
    }

    // Set dato environment if custom environment is provided
    if (options.dato.environment) {
      if (!nextConfig.env) nextConfig.env = {}
      process.env.HASHI_DATO_ENVIRONMENT = options.dato.environment
      nextConfig.env.HASHI_DATO_ENVIRONMENT = options.dato.environment
    }

    // the difference between defaults and permanents are that defaults will
    // be overridden by user land configuration, whereas permanents are always
    // tacked on to the end and therefore can't be overidden
    // header overriding docs: https://nextjs.org/docs/api-reference/next.config.js/headers#header-overriding-behavior
    const defaultHeaders = []
    const permanentHeaders = [
      {
        source: '/:path*{/}?', // https://github.com/vercel/next.js/issues/14930
        headers: [
          {
            key: 'X-Frame-Options',
            value: 'SAMEORIGIN',
          },
        ],
      },
    ]

    // support tipBranch config, which activates noindex HTTP headers
    // MKTG-030 - https://docs.google.com/document/d/1AlBW3DPT7D54vDQa0P6QBMjTKqssJFPDz1CuthkI5oM/edit#
    if (
      nextConfig.tipBranch &&
      process.env.VERCEL_GIT_COMMIT_REF == nextConfig.tipBranch
    ) {
      defaultHeaders.push({
        source: '/:path*{/}?',
        headers: [{ key: 'X-Robots-Tag', value: 'noindex' }],
      })
    }
    // if there are user land headers, combine them with defaults & permanents
    if (nextConfig.headers) {
      nextConfig.headers = nextConfig.headers().then((res) => {
        return [...defaultHeaders, ...res, ...permanentHeaders]
      })
    } else {
      // else, just add defaults and permanent headers and resolve the promise
      nextConfig.headers = Promise.resolve([
        ...defaultHeaders,
        ...permanentHeaders,
      ])
    }

    return chain.reduce((acc, next) => next(acc), nextConfig)
  }
}
